# --------------------------------------------------------------------
#              ______________________________________
#             |                                      |
#             | MySQLNews 1.0 by                     |
#             |   Matt Schultz (QMX Software)        |
#             |                                      |
#             | IRC:   irc.qmx-software.com #help    |
#             | email: matt@qmx-software.com         |
#             | www:   www.qmx-software.com          |
#             | AIM:   inuyasha2964                  |
#             | YIM:   inuyasha296                   |
#             | MSN:   isangaft220@hotmail.com       |
#             |______________________________________|
#
#              ______________________________________
#             |                                      |
#             |    This file is distributed under    |
#             |            the GNU GPL               |
#             | http://www.gnu.org/copyleft/gpl.html |
#             |______________________________________|
# --------------------------------------------------------------------
# Usage
# --------------------------------------------------------------------
# set the channel you are using this on to +newsengine
#
# !news -  Querys the last 5 lines of news from the database
# !addnews <text> - Adds news to the database (+v only)
# !delnews <#> -  Removes news from the database. You can only 
#                 delete the news if you are a bot/channel master, 
#                 or if you're the person who added it.
# --------------------------------------------------------------------
# Dependencies
# --------------------------------------------------------------------
# This script is dependant on:
#       Eggdrop (duh) (http://www.eggheads.org)
#       TCL (http://www.tcl.tk)
#       MySQL (http://www.mysql.com)
#       libMySQLTCL (http://www.xdobry.de/mysqltcl)
#
# --------------------------------------------------------------------
# INFO
# --------------------------------------------------------------------
# This is a script to add a MySQL based news system to Eggdrop.
#
# This script has been tested on:
#   RedHat Fedora Core 3 Linux running on an i686 at 450mHz
#   with the 2.6.12-1.1381_FC3 kernel build
#     with:
#       Eggdrop 1.6.17
#       TCL 8.4
#       MySQL 4.1.14-Max
#       MySQLTCL 3.01
#
# WARNING! I am not sure how this works with MySQL 5.0.
# If you plan on using this with MySQL 5.0, you do so AT YOUR OWN RISK!
# I will help in any way I can, but I am not responsible for any damages.
#
# special thanks to:
#
#   Karosh for helping me with a few TCL commands I couldn't think of
#
#   Nenearen and Stydracos for helping me with a few MySQL commands
#
# --------------------------------------------------------------------
# SQL
# --------------------------------------------------------------------
# run this in your bot's database to create the table:
#
# CREATE TABLE news (
#   id bigint(20) NOT NULL auto_increment,
#   nick varchar(20) DEFAULT '' NOT NULL,
#   host varchar(100) DEFAULT '' NOT NULL,
#   news varchar(255) DEFAULT '' NOT NULL,
#   channel varchar(50) DEFAULT '' NOT NULL,
#   timestamp bigint(20),
#   PRIMARY KEY (id)
# );
# --------------------------------------------------------------------
# VARIABLES
# you have to change these
# --------------------------------------------------------------------

# REMOVE THIS or your bot will terminate on load
die "NewsEngine: RTFM"

# MySQL hostname
set db(host) "localhost"

# MySQL user
set db(id) "CHANGE ME"

# MySQL password
set db(pass) "CHANGE ME"

# MySQL database
set db(dbase) "CHANGE ME"

# load the mysqltcl interface
load PATH/TO/libmysqltcl.so

# --------------------------------------------------------------------
# Bindings (these shouldn't have to be changed)
#    !delnews is set to opers only
#    !addnews is set to voice and opers
#    !news is set to everyone
# --------------------------------------------------------------------
bind pub "m|ov" !addnews news_add
bind pub "m|o" !delnews news_delete
bind pub "-|-" !news news

# a user with this flag(s) can't use this script at all
set news_noflags "Q|Q"
# --------------------------------------------------------------------
# MAGIC BORDER
# Don't cross this line unless you feel prepared to code
# --------------------------------------------------------------------

# --------------------------------------------------------------------
# Database Connection Handle
# --------------------------------------------------------------------
set db_handle [mysqlconnect -host $db(host) -user $db(id) -password $db(pass) -db $db(dbase)]


set news_version "1.0"
setudef flag newsengine

# --------------------------------------------------------------------
# Procedures
# --------------------------------------------------------------------

# --------------------------------------------------------------------
# news_add
# !addnews <text>
#   Adds news to the database
# --------------------------------------------------------------------
proc news_add  { nick host handle channel text } {
  global db_handle news_noflags

  if {![channel get $channel newsengine]} {
    return 0
  }

  if [matchattr $handle $news_noflags] { return 0 }

  if {($handle == "") || ($handle == "*")} {
    set handle $nick
  }

  set sql "INSERT INTO news VALUES(null, "
  append sql "'$handle', "
  append sql "'$nick!$host', "
  set text [mysqlescape $text]
  append sql "'$text', "
  append sql "'$channel', "
  append sql "'[clock seconds]')"
  
  putloglev d * "News: executing $sql"
  
  set result [mysqlexec $db_handle $sql]
  if {$result != 1} {
    putlog "An error occurred while executing the SQL command"
  } else {
    set id [mysqlinsertid $db_handle]
    puthelp "PRIVMSG $channel :News \002$id\002 added, Thank you"
  }
}

# --------------------------------------------------------------------
# news_delete
# !delnews <id>
#   Removes news from the database. You can only delete the news if you
#   are a bot/channel master, or if you're the person who added it.
# --------------------------------------------------------------------
proc news_delete  { nick host handle channel text } {
  global db_handle news_noflags

  if {![channel get $channel newsengine]} {
    return 0
  }

  if [matchattr $handle $news_noflags] { return 0 }
  
  set id [mysqlescape $text]
  if {![matchattr $handle m|m $channel]} {
    set sql "SELECT nick FROM news WHERE id='text'"
    putloglev d * "NewsEngine: executing $sql"
    
    set result [mysqlquery $db_handle $sql]
    set owner [lindex [mysqlnext $result] 0]
    mysqlendquery $result
    if {$owner != $handle} {
      putchan "NOTICE $nick :You cannot delete that news line."
      return 0
    }
  }

  set sql "DELETE FROM news WHERE id='$text'"
  putloglev d * "NewsEngine: executing $sql"
  
  set result [mysqlexec $db_handle $sql]
  if {$result != 1} {
    puthelp "PRIVMSG $channel :An error occured deleting $text from the news table"
    return 0
  } else {
    puthelp "PRIVMSG $channel :Deleted news line $text"
  }
}


# ----------------------------------------------------------------------
# news
# !news
#   Querys the last 5 lines of news from the database or the news line 
#   number entered
# ----------------------------------------------------------------------
proc news { nick host handle channel text } {
  global db_handle news_noflags

  if {![channel get $channel newsengine]} {
    return 0
  }

  if [matchattr $handle $news_noflags] { return 0 }

  set x 5
  while {$x > 0} {
    set y [expr $x - 1]
    set sql "SELECT * FROM news ORDER BY id desc limit $y,1"
    putloglev d * "NewsEngine: executing $sql"

    set result [mysqlquery $db_handle $sql]
    
    if {$x == 5} {
      puthelp "PRIVMSG $channel :NEWS:"
    }

    if {[set row [mysqlnext $result]] != ""} {
      set id [lindex $row 0]
      set news [lindex $row 3]
      set by [lindex $row 1]
      set when [clock format [lindex $row 5] -format "%d %b %Y %H:%M"]
      set chan [lindex $row 4]
      if {$chan != $channel} {
        puthelp "PRIVMSG $channel :\[\002$id\002\] $when from $by on $chan: $news"
      } else {
        puthelp "PRIVMSG $channel :\[\002$id\002\] $when from $by: $news"
      }
    } else {
      if {$x == 1} {
        puthelp "PRIVMSG $channel :Couldn't find any news (try adding news first!)"
      }
    }
    if {$x == 1} {
      puthelp "PRIVMSG $channel :End of news"
    }
    mysqlendquery $result
    set x [expr $x - 1]
  }
}
# EOF